/* word.c   by Michael Thorpe   2005-07-14 */

#include <stdlib.h>
#include <string.h>
#include "lisp.h"

static void drop_word(obj *o) {
	free(o->value.s.data);
}

static obj *word_evaluate(scope *scope,obj *in) {
	return(lookup(scope,in));
}

static obj *word_tostring(obj *in) {
	obj *out;

	out=newstring(in->value.s.data,in->value.s.len);
	decref(in);
	return(out);
}

obj *newword(const char *data,unsigned long len) {
	obj *o;
	char *s;

	s=(char *)malloc(len);
	if(!s)
		return(0);
	o=newobj(&word_objtype);
	if(!o) {
		free(s);
		return(0);
	}
	o->value.s.data=s;
	o->value.s.len=len;
	if(data)
		memcpy(o->value.s.data,data,len);
	else
		memset(o->value.s.data,0,len);
	return(o);
}

int real_eqv(obj *o1,obj *o2) {
	if(o1==o2)
		return(1);
	if(o1->type != o2->type)
		return(0);
	if(o1->type==&integer_objtype) {
		if(o1->value.i==o2->value.i)
			return(1);
	} else if(o1->type==&string_objtype || o1->type==&word_objtype) {
		if(o1->value.s.len != o2->value.s.len)
			return(0);
		if(!memcmp(o1->value.s.data,o2->value.s.data,o1->value.s.len))
			return(1);
	}
	return(0);
}

objtype word_objtype={
	name: "word",
	tostring: word_tostring,
	dropobj: drop_word,
	evaluate: word_evaluate
};

